// generated list of objects created with gen_external.py

#include <Python.h>

extern PyTypeObject Py_dctType;
extern PyTypeObject Py_mfccType;
extern PyTypeObject Py_specdescType;
extern PyTypeObject Py_tssType;
extern PyTypeObject Py_pitchType;
extern PyTypeObject Py_onsetType;
extern PyTypeObject Py_tempoType;
extern PyTypeObject Py_notesType;
extern PyTypeObject Py_samplerType;
extern PyTypeObject Py_wavetableType;

int generated_objects ( void );
void add_generated_objects( PyObject *m );
